% Quellcode der Level-2 MATLAB S-Funktion pt1sfun_m2.m, 
% fr das MATLAB-Release R2011a (Matlab 7.12, Simulink 7.7)

function pt1sfun_m2(block)

% PT1SFUN_M2 modelliert das Verhalten eines PT1-Gliedes. Die Zeit-
% konstante T wird als Parameter bergeben
                          
setup(block);
  
%endfunction

function setup(block) 

  %%%% Beginn Block-Info %%%%  
 
  % Eigenschaften (Anzahl I/Os, Dimension, Datentyp, Komplexitt, 
  % Direct Feedthrough Y/N etc.) sollen dynamisch bezogen werden 
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
  
  % Anzahl der bergebenen Parameter
  block.NumDialogPrms     = 1;              % Nur T wird bergeben
  block.DialogPrmsTunable = {'Nontunable'}; % T soll whrend der Simulation
                                            % nicht verstellbar sein

  % Anzahl der kontinuierlichen Zustandsgren (kann nicht dynamisch
  % bezogen werden)
  block.NumContStates = 1;                                            
                                            
  % Abtastzeit und Offset werden zu Null gesetzt, da System
  % zeitkontinuierlich gerechnet wird (knnte auch dynamisch bezogen werden)
  block.SampleTimes = [0 0];
  
  %%%% Ende Block-Info %%%%

  % Bentigte Callback Methoden in dieser S-Funktion
  block.RegBlockMethod('CheckParameters', @CheckPrms);
  block.RegBlockMethod('PostPropagationSetup', @DoPostPropSetup); 
  block.RegBlockMethod('InitializeConditions', @InitializeConditions);
  block.RegBlockMethod('Outputs', @Outputs);
  block.RegBlockMethod('Derivatives', @Derivatives);
   
%endfunction

% berprfe bergebenen Parameter 
function CheckPrms(block)

  if block.DialogPrm(1).Data <= 0 
    error('Die Zeitkonstante T muss grer Null gewhlt werden');
  end
  
%endfunction

% Definiere den Data Type Work Vector DWork als "globale" (d.h. von 
% allen Funktionen)lesbare Variable; DWork dient als Speicher fr die
% System-Matrizen.
function DoPostPropSetup(block)
  block.NumDworks = 1;
  
  block.Dwork(1).Name            = 'Systemmmatrizen';% muss angegeben werden
  block.Dwork(1).Dimensions      = 4;                % A,B,C,D
  block.Dwork(1).DatatypeID      = 0;                % double
  block.Dwork(1).Complexity      = 'Real';           % reell

%endfunction

% Definiere System-Matrizen als Elemente von DWork 
% und setze Anfangswert der Zustandsgre
function InitializeConditions(block)

  % Zustandsmatrix A (hier skalar)
  block.Dwork(1).Data(1) = [-1/block.DialogPrm(1).Data]; 
  % Eingangsmatrix B (hier skalar)
  block.Dwork(1).Data(2) = [ 1/block.DialogPrm(1).Data]; 
  % Ausgangsmatrix C (hier skalar)
  block.Dwork(1).Data(3) = [ 1 ];                        
  % Durchschaltmatrix D (hier skalar und gleich Null, da sich beim PT1
  % der Eingang nicht direkt auf den Ausgang auswirkt)
  block.Dwork(1).Data(4) = [ 0 ];                        
  
  block.ContStates.Data(1) = zeros(size(block.Dwork(1).Data(1),1),1);
  
%endfunction

% Berechne Systemausgang zu y = C*x+D*u
function Outputs(block)

  x = block.ContStates.Data(1);
  u = block.InputPort(1).Data;
  C = block.Dwork(1).Data(3);        % Ausgangsmatrix C
  D = block.Dwork(1).Data(4);        % Durchschaltmatrix D 
  block.OutputPort(1).Data = C*x + D*u;
  
%endfunction

% Berechne Zustandsableitung zu x' = A*x+B*u
function Derivatives(block)

   x = block.ContStates.Data(1);
   u = block.InputPort(1).Data;
   A = block.Dwork(1).Data(1); % Zustandsmatrix A  
   B = block.Dwork(1).Data(2); % Eingangsmatrix B

block.Derivatives.Data(1) = A*x + B*u;

%endfunction
